@@ -84,7 +84,7 @@ def compute_data_mean_std(
 
 if __name__ == '__main__':
 
-    def main(progress=True, num_workers=0, batch_size=256):  # try changing workers to zero on MacOS
+    def main(progress=True, num_workers=32, batch_size=2048):  # try changing workers to zero on MacOS
         from disent.dataset import data
         from docs.examples.extend_experiment.code import groundtruth__xyblocks as edat
         from disent.dataset.transform import ToImgTensorF32
@@ -97,6 +97,8 @@ if __name__ == '__main__':
             data.SmallNorb64Data,
             data.DSpritesData,
             data.Shapes3dData,
+            data.SpritesData,
+            data.SpritesAllData,
             # groundtruth -- impl synthetic
             data.XYObjectData,
             data.XYObjectShadedData,
@@ -208,6 +210,7 @@ if __name__ == '__main__':
 #     mean: [0.046146392822265625]
 #     std: [0.2096506119375896]
 
+# TODO -- REGENERATE:
 # Mpi3dData - mpi3d_toy - {'subset': 'toy', 'in_memory': True}:
 #     mean: [0.22681593831231503, 0.22353985202496676, 0.22666059934624702]
 #     std: [0.07854112062669572, 0.07319301658077378, 0.0790763900050426]
